New logging categories added to allow differentiation between
[rsync/rsync.git] / main.c
diff --git a/main.c b/main.c
index 188a99e..4c040b9 100644 (file)
--- a/main.c
+++ b/main.c
@@ -21,6 +21,7 @@
  */
 
 #include "rsync.h"
+#include "ifuncs.h"
 #include "io.h"
 #if defined CONFIG_LOCALE && defined HAVE_LOCALE_H
 #include <locale.h>
@@ -40,7 +41,7 @@ extern int remove_source_files;
 extern int need_messages_from_generator;
 extern int kluge_around_eof;
 extern int do_stats;
-extern int log_got_error;
+extern int got_xfer_error;
 extern int module_id;
 extern int copy_links;
 extern int copy_dirlinks;
@@ -49,6 +50,7 @@ extern int preserve_hard_links;
 extern int protocol_version;
 extern int file_total;
 extern int recurse;
+extern int xfer_dirs;
 extern int protect_args;
 extern int relative_paths;
 extern int sanitize_paths;
@@ -73,6 +75,9 @@ extern char *password_file;
 extern char curr_dir[MAXPATHLEN];
 extern struct file_list *first_flist;
 extern struct filter_list_struct server_filter_list;
+#ifdef ICONV_OPTION
+extern iconv_t ic_send;
+#endif
 
 int local_server = 0;
 int daemon_over_rsh = 0;
@@ -267,9 +272,10 @@ static void output_summary(void)
                        "sent %s bytes  received %s bytes  %s bytes/sec\n",
                        human_num(total_written), human_num(total_read),
                        human_dnum((total_written + total_read)/(0.5 + (endtime - starttime)), 2));
-               rprintf(FINFO, "total size is %s  speedup is %.2f\n",
+               rprintf(FINFO, "total size is %s  speedup is %.2f%s\n",
                        human_num(stats.total_size),
-                       (double)stats.total_size / (total_written+total_read));
+                       (double)stats.total_size / (total_written+total_read),
+                       dry_run ? " (DRY RUN)" : "");
        }
 
        fflush(stdout);
@@ -435,6 +441,7 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char **remote_argv, in
 
        if (read_batch) {
                int from_gen_pipe[2];
+               set_allow_inc_recurse();
                if (fd_pair(from_gen_pipe) < 0) {
                        rsyserr(FERROR, errno, "pipe");
                        exit_cleanup(RERR_IPC);
@@ -443,29 +450,58 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char **remote_argv, in
                *f_out_p = from_gen_pipe[1];
                *f_in_p = batch_fd;
                ret = -1; /* no child pid */
+#ifdef ICONV_CONST
+               setup_iconv();
+#endif
        } else if (local_server) {
                /* If the user didn't request --[no-]whole-file, force
                 * it on, but only if we're not batch processing. */
                if (whole_file < 0 && !write_batch)
                        whole_file = 1;
+               set_allow_inc_recurse();
                ret = local_child(argc, args, f_in_p, f_out_p, child_main);
+#ifdef ICONV_CONST
+               setup_iconv();
+#endif
        } else {
+#ifdef ICONV_CONST
+               setup_iconv();
+#endif
                if (protect_args) {
-                       char *save_opts1, *save_opts2;
-                       for (i = 0; strcmp(args[i], "--server") != 0; i++) {}
-                       save_opts1 = args[i+1];
-                       save_opts2 = args[i+2];
-                       args[i+1] = "-s";
-                       args[i+2] = NULL;
+                       int fd;
+#ifdef ICONV_OPTION
+                       int convert = ic_send != (iconv_t)-1;
+                       xbuf outbuf, inbuf;
+
+                       if (convert)
+                               alloc_xbuf(&outbuf, 1024);
+#endif
+
                        ret = piped_child(args, f_in_p, f_out_p);
-                       args[i] = args[i-1]; /* move command name over --server */
-                       args[i+1] = save_opts1;
-                       args[i+2] = save_opts2;
-                       while (args[i]) {
-                               write_sbuf(*f_out_p, args[i++]);
-                               write_byte(*f_out_p, 0);
-                       }
-                       write_byte(*f_out_p, 0);
+
+                       for (i = 0; args[i]; i++) {} /* find first NULL */
+                       args[i] = "rsync"; /* set a new arg0 */
+                       if (verbose > 1)
+                               print_child_argv("protected args:", args + i + 1);
+                       fd = *f_out_p;
+                       do {
+#ifdef ICONV_OPTION
+                               if (convert) {
+                                       INIT_XBUF_STRLEN(inbuf, args[i]);
+                                       iconvbufs(ic_send, &inbuf, &outbuf,
+                                                 ICB_EXPAND_OUT | ICB_INCLUDE_BAD | ICB_INCLUDE_INCOMPLETE);
+                                       outbuf.buf[outbuf.len] = '\0';
+                                       write_buf(fd, outbuf.buf, outbuf.len + 1);
+                                       outbuf.len = 0;
+                               } else
+#endif
+                                       write_buf(fd, args[i], strlen(args[i]) + 1);
+                       } while (args[++i]);
+                       write_byte(fd, 0);
+#ifdef ICONV_OPTION
+                       if (convert)
+                               free(outbuf.buf);
+#endif
                } else
                        ret = piped_child(args, f_in_p, f_out_p);
        }
@@ -559,7 +595,8 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
                        exit_cleanup(RERR_FILEIO);
                }
 
-               if (strcmp(flist->files[0]->basename, ".") == 0)
+               if (flist->high >= flist->low
+                && strcmp(flist->files[flist->low]->basename, ".") == 0)
                        flist->files[0]->flags |= FLAG_DIR_CREATED;
 
                if (verbose)
@@ -687,7 +724,7 @@ static void do_server_sender(int f_in, int f_out, int argc, char *argv[])
        argc--;
        argv++;
 
-       if (argc == 0 && (recurse || list_only)) {
+       if (argc == 0 && (recurse || xfer_dirs || list_only)) {
                argc = 1;
                argv--;
                argv[0] = ".";
@@ -889,10 +926,10 @@ static void do_server_recv(int f_in, int f_out, int argc, char *argv[])
        if (sanitize_paths) {
                char **dir;
                for (dir = basis_dir; *dir; dir++) {
-                       *dir = sanitize_path(NULL, *dir, NULL, curr_dir_depth, NULL);
+                       *dir = sanitize_path(NULL, *dir, NULL, curr_dir_depth);
                }
                if (partial_dir) {
-                       partial_dir = sanitize_path(NULL, partial_dir, NULL, curr_dir_depth, NULL);
+                       partial_dir = sanitize_path(NULL, partial_dir, NULL, curr_dir_depth);
                }
        }
        fix_basis_dirs();
@@ -933,9 +970,6 @@ void start_server(int f_in, int f_out, int argc, char *argv[])
 
        io_set_sock_fds(f_in, f_out);
        setup_protocol(f_out, f_in);
-#ifdef ICONV_CONST
-       setup_iconv();
-#endif
 
        if (protocol_version >= 23)
                io_start_multiplex_out();
@@ -970,9 +1004,6 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
 
        io_set_sock_fds(f_in, f_out);
        setup_protocol(f_out,f_in);
-#ifdef ICONV_CONST
-       setup_iconv();
-#endif
 
        /* We set our stderr file handle to blocking because ssh might have
         * set it to non-blocking.  This can be particularly troublesome if
@@ -1030,8 +1061,10 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
                        io_start_multiplex_out();
        }
 
-       if (argc == 0)
+       if (argc == 0) {
                list_only |= 1;
+               xfer_dirs |= 1;
+       }
 
        send_filter_list(read_batch ? -1 : f_out);
 
@@ -1108,15 +1141,15 @@ static int start_client(int argc, char *argv[])
        if (!read_batch) { /* for read_batch, NO source is specified */
                char *path = check_for_hostspec(argv[0], &shell_machine, &rsync_port);
                if (path) { /* source is remote */
-                       char *dummy1;
-                       int dummy2;
+                       char *dummy_host;
+                       int dummy_port = 0;
                        *argv = path;
                        remote_argv = argv;
                        remote_argc = argc;
                        argv += argc - 1;
                        if (argc == 1 || **argv == ':')
                                argc = 0; /* no dest arg */
-                       else if (check_for_hostspec(*argv, &dummy1, &dummy2)) {
+                       else if (check_for_hostspec(*argv, &dummy_host, &dummy_port)) {
                                rprintf(FERROR,
                                        "The source and destination cannot both be remote.\n");
                                exit_cleanup(RERR_SYNTAX);
@@ -1143,7 +1176,8 @@ static int start_client(int argc, char *argv[])
                                static char *dotarg[1] = { "." };
                                p = dotarg[0];
                                remote_argv = dotarg;
-                               list_only = 1;
+                               list_only |= 1;
+                               xfer_dirs |= 1;
                        }
                        remote_argc = 1;
 
@@ -1179,29 +1213,46 @@ static int start_client(int argc, char *argv[])
        }
 
        if (am_sender) {
-               char *dummy1;
-               int dummy2;
+               char *dummy_host;
+               int dummy_port = rsync_port;
                int i;
                /* For local source, extra source args must not have hostspec. */
                for (i = 1; i < argc; i++) {
-                       if (check_for_hostspec(argv[i], &dummy1, &dummy2)) {
+                       if (check_for_hostspec(argv[i], &dummy_host, &dummy_port)) {
                                rprintf(FERROR, "Unexpected remote arg: %s\n", argv[i]);
                                exit_cleanup(RERR_SYNTAX);
                        }
                }
        } else {
+               char *dummy_host;
+               int dummy_port = rsync_port;
                int i;
/* For remote source, any extra source args must be ":SOURCE" args. */
                for (i = 1; i < remote_argc; i++) {
-                       if (*remote_argv[i] != ':') {
+                       char *arg = check_for_hostspec(remote_argv[i], &dummy_host, &dummy_port);
+                       if (!arg) {
                                rprintf(FERROR, "Unexpected local arg: %s\n", remote_argv[i]);
                                rprintf(FERROR, "If arg is a remote file/dir, prefix it with a colon (:).\n");
                                exit_cleanup(RERR_SYNTAX);
                        }
-                       remote_argv[i]++;
+                       if (*dummy_host && strcmp(dummy_host, shell_machine) != 0) {
+                               rprintf(FERROR, "All source args must come from the same machine.\n");
+                               exit_cleanup(RERR_SYNTAX);
+                       }
+                       if (rsync_port != dummy_port) {
+                               if (!rsync_port || !dummy_port)
+                                       rprintf(FERROR, "All source args must use the same hostspec format.\n");
+                               else
+                                       rprintf(FERROR, "All source args must use the same port number.\n");
+                               exit_cleanup(RERR_SYNTAX);
+                       }
+                       remote_argv[i] = arg;
                }
-               if (argc == 0)
+               if (argc == 0) {
                        list_only |= 1;
+                       xfer_dirs |= 1;
+               }
        }
 
        if (daemon_over_rsh < 0)
@@ -1259,7 +1310,7 @@ static RETSIGTYPE sigusr2_handler(UNUSED(int val))
        if (!am_server)
                output_summary();
        close_all();
-       if (log_got_error)
+       if (got_xfer_error)
                _exit(RERR_PARTIAL);
        _exit(0);
 }
@@ -1437,12 +1488,16 @@ int main(int argc,char *argv[])
        if (write_batch < 0)
                dry_run = 1;
 
-       if (am_daemon && !am_server)
+       if (am_server) {
+#ifdef ICONV_CONST
+               setup_iconv();
+#endif
+       } else if (am_daemon)
                return daemon_main();
 
        if (am_server && protect_args) {
                char buf[MAXPATHLEN];
-               protect_args = 0;
+               protect_args = 2;
                read_args(STDIN_FILENO, NULL, buf, sizeof buf, 1, &argv, &argc, NULL);
                if (!parse_arguments(&argc, (const char ***) &argv, 1)) {
                        option_error();
@@ -1179,29 +1213,46 @@ static int start_client(int argc, char *argv[])
                int i;
-               /* For remote source, any extra source args must be ":SOURCE" args. */
+               /* For remote source, any extra source args must have either
+                * the same hostname or an empty hostname. */
                for (i = 1; i < remote_argc; i++) {